_G.LibLinq_1_0_Loader(function(LibLinq)
	local _G = _G
	local assert = _G.assert

	local make_weak_keyed_table = assert(LibLinq.Utilities.make_weak_keyed_table)
	local check = assert(LibLinq.Utilities.check)
	local tostring2 = assert(LibLinq.Utilities.tostring2)
	local TryFinally = assert(LibLinq.Utilities.TryFinally)
	local safe_dispose = assert(LibLinq.Utilities.safe_dispose)

	local newproxy = assert(_G.newproxy)
	local getmetatable = assert(_G.getmetatable)
	local type = assert(_G.type)
	local table_concat = assert(_G.table.concat)
	local tostring = assert(_G.tostring)

	local Enumerator = assert(LibLinq.Enumerator)

	--- A class that exposes an enumerator and supports simiple iteration as well as helper methods.
	local Enumerable = LibLinq.Enumerable or {}
	do
		LibLinq.Enumerable = Enumerable
		if not Enumerable.prototype then
			Enumerable.prototype = {}
		end
		local Enumerable_proxy = newproxy(true)
		local Enumerable_mt = getmetatable(Enumerable_proxy)
		Enumerable_mt.__index = Enumerable.prototype

		local get_enumerators = make_weak_keyed_table(Enumerable.__get_enumerators)
		Enumerable.__get_enumerators = get_enumerators

		--- Return the enumerator for the current Enumerable
		-- @return an Enumerator
		function Enumerable.prototype:GetEnumerator()
			return get_enumerators[self](self)
		end

		--- Construct and return a new Enumerable
		-- @param enumerator_creator a function that will return an enumerator that will be called when :GetEnumerator() is called.
		-- @return The new Enumerable
		function Enumerable.New(get_enumerator)
			check(1, get_enumerator, 'function')

			local self = newproxy(Enumerable_proxy)

			get_enumerators[self] = get_enumerator

			return self
		end
	
		--- Return whether the provided object is an Enumerable
		-- @param obj the object to check
		-- @return whether the object inherits from or is an Enumerable
		function Enumerable.IsEnumerable(obj)
			local obj_type = type(obj)
			if obj_type ~= 'userdata' then
				return false
			end

			while obj do
				local mt = getmetatable(obj)
				if not mt then
					return false
				elseif mt.__index == Enumerable.prototype then
					return true
				end

				obj = mt.__index
			end
			return false
		end

		function Enumerable_mt:__tostring()
			return self:ToString()
		end
	end

	--- Convert the enumerable to a simple lua table.
	-- The implementation can vary the result, a list might return a numerically indexed table,
	-- whereas a set might return a table with all values as true.
	-- A dictionary will probably return a standard dictionary-style table.
	-- Not all values may show up in the table, especially in cases where nil is involved.
	-- @return a lua table
	-- @usage Enumerable.From({ 'a', 'b', 'c' }):ToTable()[2] == 'b'
	function Enumerable.prototype:ToTable()
		check(1, self, 'userdata')
	
		local t = {}
		self:ForEach(function(x, i)
			t[i] = x
		end)
		return t
	end

	--- Return a string representation of this Enumerable.
	-- This should only show the first 10 elements of the Enumerable, after which it follows with an ellipsis (...).
	-- @return a string
	function Enumerable.prototype:ToString()
		check(1, self, 'userdata')
	
		local t = {}
		t[#t+1] = '['
		self:ForEach(function(value, i)
			if i > 1 then
				t[#t+1] = ', '
			end
			if i >= 11 then
				t[#t+1] = '...'
				return false
			end
			t[#t+1] = tostring2(value)
		end)
		t[#t+1] = ']'
		return table_concat(t)
	end

	local function unpack_helper(enumerator)
		if enumerator:MoveNext() then
			return enumerator:Current(), unpack_helper(enumerator)
		else
			return
		end
	end

	--- Unpack the contents of the enumerable to a lua argument tuple
	-- @return zero or more values
	-- @usage a, b, c = Enumerable.From({ 1, 2, 3 }):Unpack()
	function Enumerable.prototype:Unpack()
		check(1, self, 'userdata')

		local enumerator = self:GetEnumerator()

		return TryFinally(function()
			return unpack_helper(enumerator)
		end, function()
			safe_dispose(enumerator)
		end)
	end

	--- Concatenate the values of the enumerable into a single string
	-- @param separator optional: the separator to use, defaults to an empty string
	-- @param selector optional: a function to select over the enumerable
	-- @return a string
	-- @usage Enumerable.From({ 1, 2, 3 }):StringJoin() == "123"
	-- @usage Enumerable.From({ 1, 2, 3 }):StringJoin(", ") == "1, 2, 3"
	-- @usage Enumerable.From({ 1, 2, 3 }):StringJoin(", ", function(x) return x*x end) == "1, 4, 9"
	function Enumerable.prototype:StringJoin(separator, selector)
		check(1, self, 'userdata')
		check(2, separator, 'string', 'nil')
		check(3, selector, 'function', 'nil')

		if selector then
			return self:Select(selector):StringJoin(separator)
		end

		local t = {}
		self:ForEach(function(item)
			if item ~= nil then
				t[#t+1] = tostring(item)
			end
		end)
		return table_concat(t, separator or '')
	end

	do
		local not_nil_filter = function(value) return value ~= nil end
		--- Return all non-nil elements
		-- @return an Enumerable
		function Enumerable.prototype:WhereNotNil()
			check(1, self, 'userdata')

			return self:Where(not_nil_filter)
		end
	end
end)